@@ -8,14 +8,15 @@ from asyncio import CancelledError
 from datetime import datetime, timezone
 from typing import Awaitable, Optional, Dict, List, Union, Tuple, Any
 
-from hopeit.server.imports import find_event_handler
-from hopeit.server.steps import split_event_stages, event_and_step, extract_module_steps, effective_steps
+from hopeit.server.imports import find_datobject_type, find_event_handler
+from hopeit.server.steps import split_event_stages, event_and_step, extract_module_steps, \
+    effective_steps
 from hopeit.toolkit import auth
 from hopeit.app.config import AppConfig, EventSettings, EventType, ReadStreamDescriptor, EventDescriptor, \
     StreamQueue, StreamQueueStrategy
 from hopeit.app.context import EventContext, PostprocessHook, PreprocessHook
 from hopeit.app.client import register_app_connections, stop_app_connections
-from hopeit.dataobjects import EventPayload
+from hopeit.dataobjects import DataObject, EventPayload
 from hopeit.server.config import ServerConfig
 from hopeit.server.events import EventHandler, get_event_settings, get_runtime_settings
 from hopeit.streams import stream_auth_info, StreamEvent, StreamOSError, StreamManager
@@ -380,7 +381,7 @@ class AppEngine:
                     consumer_group=stream_info.consumer_group
                 )
 
-            datatypes = self._find_stream_datatype_handlers(event_name)
+            datatypes = self._find_stream_datatype_handlers(event_name, event_config)
             log_info['name'] = stream_info.name
             log_info['consumer_group'] = stream_info.consumer_group
             assert not self._running[event_name].locked(), f"Event already running {event_name}"
@@ -452,7 +453,7 @@ class AppEngine:
         if previous_context is None:
             track_ids = {
                 'track.request_id': str(uuid.uuid4()),
-                'track.request_ts': datetime.now().astimezone(timezone.utc).isoformat()
+                'track.request_ts': datetime.now(tz=timezone.utc).isoformat()
             }
         else:
             track_ids = previous_context.track_ids
@@ -493,7 +494,8 @@ class AppEngine:
                         extra=extra(prefix='service.', **log_info))
             await asyncio.sleep(wait)
         logger.info(__name__, "Starting service...", extra=extra(prefix='service.', **log_info))
-        impl = find_event_handler(app_config=self.app_config, event_name=event_name)
+        event_config = self.effective_events[event_name]
+        impl = find_event_handler(app_config=self.app_config, event_name=event_name, event_info=event_config)
         service_handler = getattr(impl, '__service__')
         assert service_handler is not None, \
             f"{event_name} must implement method `__service__(context) -> Spawn[...]` to run as a service"
@@ -547,27 +549,47 @@ class AppEngine:
 
     @staticmethod
     def _config_effective_events(app_config: AppConfig) -> Dict[str, EventDescriptor]:
+        """
+        Return effective events computed from user app config.
+
+        Effective events could be result of splitting a single event in stages,
+        using the "SHUFFLE" keyword, that will internaally generate 2 events.
+        Or for STREAMS that implementes the `__service__` method, both a STREAM
+        and a SERVICE event will be generated.
+        """
         effective_events: Dict[str, EventDescriptor] = {}
         for event_name, event_info in app_config.events.items():
-            impl = find_event_handler(app_config=app_config, event_name=event_name)
+            impl = find_event_handler(app_config=app_config, event_name=event_name, event_info=event_info)
+            # Add events resultatn of splitting steps on SHUFFLE (stages)
             splits = split_event_stages(app_config.app, event_name, event_info, impl)
             effective_events.update(**splits)
+            # Add associated SERVICE events to streams
+            if event_info.type == EventType.STREAM and hasattr(impl, "__service__"):
+                effective_events[f"{event_name}$__service__"] = EventDescriptor(
+                    type=EventType.SERVICE,
+                    connections=event_info.connections,
+                    impl=event_info.impl,
+                )
         return effective_events
 
-    def _find_stream_datatype_handlers(self, event_name: str) -> Dict[str, type]:
+    def _find_stream_datatype_handlers(self, event_name: str, event_info: EventDescriptor) -> Dict[str, type]:
         """
         Computes a dictionary of `{datatype name: datatype class}` that event steps
         can be handle when consuming from an stream.
         """
         base_event, _ = event_and_step(event_name)
-        impl = find_event_handler(app_config=self.app_config, event_name=base_event)
+        impl = find_event_handler(app_config=self.app_config, event_name=base_event, event_info=event_info)
         all_steps = extract_module_steps(impl)
         steps = effective_steps(event_name, all_steps)
         datatypes = {}
         for _, _, step in steps:
             _, datatype, _, _ = step
             if hasattr(datatype, '__stream_event__'):
-                datatypes[datatype.__name__] = datatype
+                datatypes[f"{datatype.__module__}.{datatype.__qualname__}"] = datatype
+            elif datatype is DataObject:
+                for type_name in event_info.dataobjects:
+                    datatype = find_datobject_type(type_name)
+                    datatypes[f"{datatype.__module__}.{datatype.__qualname__}"] = datatype
         if len(datatypes) == 0:
             raise NotImplementedError(f"No data types found to read from stream in event={event_name}. "
                                       "Dataclasses must be decorated with `@dataobject` to be used in streams")
